package betir;

/**
*	This behavior moves the goalie up and down in front of the goal so
*	that he can be in a position to catch shots.  He always uses a wide view,
*	and matches the y-position of the ball.  He is also always pointed up so that
*	he is always ready to move up or down.
*/
public class GoalieUpDownBehavior extends RobocupBehavior
{
	public GoalieUpDownBehavior(RobocupEnvironment env, DatagramWrapper dg)
	{
		super(env, dg);
	}

	public boolean canFire()
	{
		return true;
	}

	public boolean inhibits(RobocupBehavior rb)
	{
		return false;
	}

	private TwoDVec m_vecBall = new TwoDVec();
	private int m_nMove = 0;
	private int m_nNumNotSeenBall = 0;
	private boolean m_bMovingUp = false;
	public boolean fire()
	{
		if(m_Env.m_aobjFlags.size() < 2)
		{
			turn(120);
			return false;
		}

		++m_nNumNotSeenBall;
		if(m_Env.m_objBall != null)
		{
			m_vecBall = m_Env.getAbsolutePos(m_Env.m_objBall);
			m_nMove = (int)(0.9f * m_Env.m_objBall.getAngleChng() * m_Env.m_objBall.getDistChng());
			m_nNumNotSeenBall = 0;
		}

		int nHeadAngle = 90;
		if(m_Env.m_objSelf.getSide() == 'r')
		{
			nHeadAngle = -90;
			m_nMove = -m_nMove;
		}

		m_vecBall.setAbsolute(m_vecBall.getX(), m_vecBall.getY() + m_nMove);

		if(withinRange(angleToAngle(90), 0, 3) && withinRange(m_Env.m_objSelf.getHeadAngle(), nHeadAngle, 3))
		{
			double dPower = 10.0 * (m_vecBall.getY() - m_Env.m_vecPosition.getY());

			if(m_bMovingUp)
				if(dPower < 0)
				{
					dPower *= 0.25;
					m_bMovingUp = false;
				}
			else
				if(dPower > 0)
				{
					dPower *= 0.25;
					m_bMovingUp = true;
				}

			if((dPower > 0 && m_Env.m_vecPosition.getY() < m_Env.m_cfgData.goal_width * 0.3f)
				|| (dPower < 0 && m_Env.m_vecPosition.getY() > -m_Env.m_cfgData.goal_width * 0.3f))
				dash((float)(dPower / m_Env.m_cfgData.dash_power_rate));

		}
		else if(!withinRange(angleToAngle(90), 0, 2) && m_Env.m_bAbsAngle)
			turn(angleToAngle(90));
		else if(m_Env.m_bAbsAngle)
			turnHead(nHeadAngle - m_Env.m_objSelf.getHeadAngle());

		if(m_nNumNotSeenBall > 5)
		{
			m_nNumNotSeenBall = 0;
			turnHead(-nHeadAngle - m_Env.m_objSelf.getHeadAngle());
		}

		changeView("wide", "high");

		return false;
	}
}