//
//  NeuralNet.java
//  betir
//
//  Created by Greg Croft on Thu Oct 18 2001.
//  Copyright (c) 2001 __MyCompanyName__. All rights reserved.
//
package betir;
import java.io.*;

public final class NeuralNet
{
//******************************************************************************************************* Neuron Classes

    private final class BoolObj		{ public boolean value; }
    private final class FloatObj	{ public float value; }
    private final class IntObj		{ public float value; }

    private final class Neuron
    {
        public 	BoolObj 	inputs[];
        public 	float		weights[];
        public 	BoolObj 	output;
    }


//******************************************************************************************************* Member Variables

    private final float	m_fThreshold = 0.5f;
    private Neuron 	m_aNeurons[];
    private int		m_nNumNetInputs;
    private float 	m_fTemp;
    private Neuron 	m_cCurrentNeuron;


//******************************************************************************************************* Constructor

    public NeuralNet()
    {

    }

//******************************************************************************************************* ParseConfigFile

    public boolean ParseConfigFile(String strFilename, BoolObj abInputs[], BoolObj abOutputs[])
    {
        DataInputStream fInput;
        try
        {
            System.out.print("Loading Neural Net File <" + strFilename + "> ...");
            fInput = new DataInputStream(new FileInputStream(strFilename));
            if(!fInput.readUTF().equals("NeuralSoccerAgent_NeuralNetConfigFile"))
                return false;	// check for file header
            abInputs = new BoolObj[fInput.readInt()]; // set inputs
            m_nNumNetInputs = abInputs.length;
            abOutputs = new BoolObj[fInput.readInt()]; // set outputs
            m_aNeurons = new Neuron[fInput.readInt() + m_nNumNetInputs]; // set number of neurons

            for(int i = 0; i < m_aNeurons.length; ++i) // create neurons & outputs
            {
                m_aNeurons[i] = new Neuron();
                m_aNeurons[i].output = new BoolObj();
            }
            for(int i = 0; i < m_nNumNetInputs; ++i) // set up dummy neurons as inputs
            {
                abInputs[i] = m_aNeurons[i].output;
            }
            for(int i = m_nNumNetInputs; i < m_aNeurons.length; ++i) // parse connections and weights for neurons
            {
                m_aNeurons[i].inputs = new BoolObj[fInput.readInt()];
                m_aNeurons[i].weights = new float[m_aNeurons[i].inputs.length];
                for(int j = 0; j < m_aNeurons[i].inputs.length; ++j)
                {
                    m_aNeurons[i].inputs[j] = m_aNeurons[fInput.readInt()].output;
                    m_aNeurons[i].weights[j] = fInput.readFloat();
                }
            }
            for(int i = m_aNeurons.length - abOutputs.length; i < m_aNeurons.length; ++i) // setup outputs
            {
                abOutputs[i - abOutputs.length] = m_aNeurons[i].output;
            }

            fInput.close();
            System.out.println("complete");
        }
        catch(IOException e)
        {
            System.err.println("File not opened properly\n" + e.toString());
            System.exit(1);
        }

        return true;
    }


//******************************************************************************************************* Propagate

    public void Propagate()
    {
        for(int i = m_nNumNetInputs; i < m_aNeurons.length; ++i)
        {
            m_fTemp = 0.0f;
            m_cCurrentNeuron = m_aNeurons[i];
            for(int j = 0; j < m_cCurrentNeuron.inputs.length; ++j)
            {
                m_fTemp += m_cCurrentNeuron.inputs[j].value ? m_cCurrentNeuron.weights[j] : 0;
            }
            m_cCurrentNeuron.output.value = m_fTemp > m_fThreshold ? true : false;
        }
    }
}
