package betir;

import java.util.*;

/**
*	This behavior shoots a goal.  It looks for both the top and bottom goal flag and then looks
*	for any enemies inbetween the flags.  Then it selects the largest open angle.
*	If there is nobody between the flags then it kicks it directly in the middle.
*	If the largest angle is between two players then it kicks it directly between them,
*	and if it is between a player and the flag then it kicks it close to the flag(goal post).
*	If it can not see the goal posts it kicks at the center of the goal.
*/
public class ShootGoalBehavior extends RobocupBehavior
{
	public ShootGoalBehavior(RobocupEnvironment env, DatagramWrapper dg)
	{
		super(env, dg);
	}

	public boolean canFire()
	{
		if(m_Env.m_bHasBall && m_Env.m_objGoal != null && m_Env.m_objGoal.getDistance() < 40)
			return true;
		return false;
	}

	public boolean inhibits(RobocupBehavior rb)
	{
		return (rb instanceof KiddySoccerBehavior || rb instanceof PassBehavior);
	}

	public boolean fire()
	{
		ObjectInfo flagB = null;
		ObjectInfo flagT = null;
		ObjectInfo obj;
		if(m_Env.m_objSelf.getSide() == 'l')
		{
			for(int i = 0; i < m_Env.m_aobjFlags.size(); i++)
			{
				obj = (ObjectInfo)m_Env.m_aobjFlags.get(i);
				if(obj.getName().equals("f g r b"))
					flagB = obj;
				else if(obj.getName().equals("f g r t"))
					flagT = obj;
			}
		}
		else
		{
			for(int i = 0; i < m_Env.m_aobjFlags.size(); i++)
			{
				obj = (ObjectInfo)m_Env.m_aobjFlags.get(i);
				if(obj.getName().equals("f g l b"))
					flagB = obj;
				else if(obj.getName().equals("f g l t"))
					flagT = obj;
			}
		}
		try{//yeah i know this is bad i'll fix it later... maybe - haven't actually noticed any exceptions - but just in case
			if(flagT != null && flagB != null)
			{
				SortedIntegerList inRange = new SortedIntegerList();
				ObjectInfo enemy;

				int nAngleTop;
				int nAngleBottom;
				nAngleTop = flagT.getAngle();
				nAngleBottom = flagB.getAngle();
				inRange.add(nAngleTop);
				inRange.add(nAngleBottom);

				//get all enemies that are in the way
				for(int i = 0; i < m_Env.m_aobjEnemies.size(); i++)
				{
					enemy =	(ObjectInfo)m_Env.m_aobjEnemies.get(i);
					if(m_Env.m_objSelf.getSide() == 'l' && nAngleBottom > enemy.getAngle() && nAngleTop < enemy.getAngle())
						inRange.add(enemy.getAngle());
					else if(m_Env.m_objSelf.getSide() == 'r' && nAngleTop > enemy.getAngle() && nAngleBottom < enemy.getAngle())
						inRange.add(enemy.getAngle());
				}
				//get the largest open angle
				int largest = 0;
				int index = 0;
				for(int i = 0; i < inRange.size() - 1; i++)
				{
					if(Math.abs(inRange.get(i) - inRange.get(i+1)) >= largest)
					{
						largest = Math.abs(inRange.get(i) - inRange.get(i+1));
						index = i;
					}
				}
				//determine the angle to kick
				int nAngle;
				if(inRange.size() < 3)//no opposition
					nAngle = (int)(Math.abs(inRange.get(index) - inRange.get(index+1))*0.5);
				else if(index == 0)//it's a flag so kick it closer to the flag
					nAngle = (int)(Math.abs(inRange.get(index) - inRange.get(index+1))*0.15);
				else if(index == inRange.size()-2)//it's a flag so kick it closer to the flag
					nAngle = (int)(Math.abs(inRange.get(index) - inRange.get(index+1))*0.85);
				else
					nAngle = (int)(Math.abs(inRange.get(index) - inRange.get(index+1))*0.5);
				nAngle += inRange.get(index);
				kick(m_Env.m_cfgData.maxpower, nAngle);
			}
			else
			{
				kick(m_Env.m_cfgData.maxpower, m_Env.m_objGoal.getAngle());
			}
		}
		catch(Exception e)
		{
			e.printStackTrace();
			System.out.println("ha ha fooled you");
		}
		return false;
	}

	private class SortedIntegerList{
		ArrayList list;

		public SortedIntegerList()
		{
			list = new ArrayList();
		}

		public void add(int nValue)
		{
			Integer val = new Integer(nValue);
			if(list.size() == 0)
				list.add(0, val);
			else
			{
				boolean bAdded = false;
				for(int i = 0; i < list.size(); i++)
				{
					if(nValue < ((Integer)list.get(i)).intValue())
					{
						list.add(i, val);
						bAdded = true;
						break;
					}
				}
				if(bAdded == false)
					list.add(val);
			}
		}

		public int get(int nIndex)
		{
			return ((Integer)list.get(nIndex)).intValue();
		}

		public int size()
		{
			return list.size();
		}
	}
}