package betir;
import java.lang.Math;

/**
*	This is a container class that is used for polar and absolute locations.
*/
public class TwoDVec
{
//****************************************************************************************************************** private data
    /**	the angle*/
    private int m_nAngle;
    /**	the magnitude*/
    private float m_fMagnitude;
    /**	the x coordinate*/
    private int m_nX;
    /**	the y coordinate*/
    private int m_nY;

//****************************************************************************************************************** constructors

    public TwoDVec() { }

    public TwoDVec(int nX, int nY, int nAngle, int fMagnitude)
	{
		m_nAngle 		= nAngle;
		m_fMagnitude 	= fMagnitude;
		m_nX 			= nX;
		m_nY 			= nY;
	}

	public TwoDVec(TwoDVec obj)
	{
		m_nAngle 		= obj.getAngle();
		m_fMagnitude 	= obj.getMagnitude();
		m_nX 			= obj.getX();
		m_nY 			= obj.getY();
	}

	public TwoDVec(TwoDVec p1, TwoDVec p2)
	{
	 	setDifference(p1, p2);
	}

//****************************************************************************************************************** set

    public void setAbsolute(int nX, int nY)
	{
		m_nX = nX;
		m_nY = nY;
	}

    public void setPolar(int nAngle, float fMagnitude)
	{
		m_nAngle = nAngle;
		m_fMagnitude = fMagnitude;
	}

	public void setAngle(int nAngle)
	{
		m_nAngle = nAngle;
	}

	public void setDifference(TwoDVec p1, TwoDVec p2)
	{
		if(p1 == null || p2 == null)
			return;

		int nX = p2.getX() - p1.getX();
		int nY = p2.getY() - p1.getY();

		m_nX = p1.getX();
		m_nY = p1.getY();

		double angle = 0;
		if(nX == 0)
			angle = (nY > 0) ? 90 : -90;
		else if(nY == 0)
			angle = (nX > 0) ? 0 : 180;
		else
		{
			angle = Math.toDegrees(Math.atan((float)(nY) / (float)(nX)));
			if(nX < 0)
				angle -= 180;
		}
		if(angle > 180) angle -= 360;
		if(angle < -180) angle += 360;
		m_nAngle = (int)(angle);
		m_fMagnitude = (float)QuickIntMath.Sqrt(nX * nX + nY * nY);
	}

//****************************************************************************************************************** get

    final public int getX() 								{ 	return m_nX; 			}
    final public int getY() 								{ 	return m_nY; 			}
    final public int getAngle() 							{ 	return m_nAngle; 		}
    final public float getMagnitude() 						{ 	return m_fMagnitude; 	}

	final public int getDistance(TwoDVec obj)
	{
		int nX = obj.getX() - m_nX;
		int nY = obj.getY() - m_nY;
		return QuickIntMath.Sqrt(nX * nX + nY * nY);
	}

//****************************************************************************************************************** rotate

	public void rotPolar(int nRot)
	{
		m_nAngle += nRot;
	}

    public void rot(int nRot)
	{
		m_nX = (int)((float)m_nX * QuickIntMath.Cos(nRot) - (float)m_nY * QuickIntMath.Sin(nRot));
		m_nY = (int)((float)m_nX * QuickIntMath.Sin(nRot) + (float)m_nY * QuickIntMath.Cos(nRot));
	}

//****************************************************************************************************************** extrapolate

	/**
	*	use this to extrapolate to the next point
	*
	*	@param decay
	*/
    public void extrapolateNext(float decay)
	{
		m_fMagnitude *= decay;
		m_nX += (int)(m_fMagnitude * QuickIntMath.Cos(m_nAngle));
		m_nY += (int)(m_fMagnitude * QuickIntMath.Sin(m_nAngle));
	}

//****************************************************************************************************************** updates

	//	use these sparingly, they copy the polar values to absolute or vice-versa

    public void updateAbsolute()
	{
		m_nX = (int)(m_fMagnitude * QuickIntMath.Cos(m_nAngle));
		m_nY = (int)(m_fMagnitude * QuickIntMath.Sin(m_nAngle));
	}

    public void updatePolar()
	{
		m_fMagnitude = QuickIntMath.Sqrt(m_nX * m_nX + m_nY * m_nY);
		m_nAngle = (int)((float)Math.toDegrees(Math.acos(m_nX / m_fMagnitude)));
	}

//****************************************************************************************************************** toString

    final public String toStringPolar()						{ return "(M:" + m_fMagnitude + ", A:" + m_nAngle + ")"; 	}
    final public String toStringAbsolute()					{ return "(X:" + m_nX + ", Y:" + m_nY + ")"; 				}
}
